@@ -217,7 +217,7 @@ int		*leftnp;	/* size of LHS */
 token_type	*rightp;	/* RHS */
 int		*rightnp;	/* size of RHS */
 {
-	int		i, j, k;
+	int		i, j;
 	int		found, found_count;
 	int		worked;
 	int		diff_sign;
@@ -390,25 +390,25 @@ zero_simp:
 				}
 				switch (op) {
 				case TIMES:
-					for (p1 = trhs; p1 < ep; p1 += 2) {
-						if (p1->kind == CONSTANT
-						    && p1->level == 1
-						    && p1->token.constant != 1.0) {
-							p1->token.constant = 1.0;
-							goto zero_simp;
-						}
-					}
-					for (b1 = &trhs[1]; b1 < ep; b1 += 2) {
-						if (b1->level == 2
-						    && b1->token.operatr == POWER
-						    && (b1 + 1)->level == 2
-						    && (b1 + 1)->kind == CONSTANT
-						    && (b1 + 1)->token.constant > 0.0) {
-							p1 = b1 + 2;
-							blt(b1, p1, (char *) ep - (char *) p1);
-							n_trhs -= 2;
-							goto zero_simp;
+					for (p1 = trhs; p1 < ep;) {
+						b1 = p1;
+						for (;; p1++) {
+							if (p1 >= ep || (p1->kind == OPERATOR && p1->level == 1)) {
+								blt(b1 + 1, p1, (char *) ep - (char *) p1);
+								n_trhs -= p1 - (b1 + 1);
+								b1->kind = CONSTANT;
+								b1->token.constant = 1.0;
+								goto zero_simp;
+							}
+							if (p1->kind != CONSTANT && p1->kind != OPERATOR
+							    && (p1->kind != VARIABLE || (p1->token.variable & VAR_MASK) > SIGN)) {
+								break;
+							}
 						}
+						p1 = b1;
+						for (p1++; p1 < ep && p1->level > 1; p1 += 2)
+							;
+						p1++;
 					}
 					break;
 				case POWER:
@@ -607,7 +607,10 @@ zero_simp:
 						}
 						if (zflag) {
 							if (op_kind == DIVIDE
-							    || (tlhs[i+1].level == 1 && tlhs[i+1].kind != CONSTANT)) {
+							    || (tlhs[i+1].kind == VARIABLE && tlhs[i+1].token.variable == v
+							    && (tlhs[i+1].level == 1
+							    || (tlhs[i+1].level == 2 && tlhs[i+2].token.operatr == POWER
+							    && tlhs[i+3].level == 2 && tlhs[i+3].kind == CONSTANT)))) {
 								op = op_kind;
 								b1 = &tlhs[i+1];
 								if (op_kind == DIVIDE)
@@ -630,17 +633,15 @@ zero_simp:
 					}
 				}
 				if ((zflag && zero_solved && op == TIMES
-				    && b1->level == 1 && b1->kind != CONSTANT)
+				    && b1->kind == VARIABLE && b1->token.variable == v
+				    && (b1->level == 1 || (b1->level == 2 && (b1 + 1)->token.operatr == POWER
+				    && (b1 + 2)->level == 2 && (b1 + 2)->kind == CONSTANT)))
 				    || op == DIVIDE) {
 					if (op == TIMES) {
 						qtries = 0;	/* might be quadratic after removing solution */
 #if	!SILENT
 						printf(_("Removing possible solution: \""));
-						k = b1 - tlhs;
-						for (j = k + 1; j < n_tlhs; j += 2)
-							if (tlhs[j].level <= 1)
-								break;
-						list_proc(b1, j - k, false);
+						list_proc(b1, 1, false);
 						printf(" = 0\".\n");
 #endif
 					} else {
@@ -741,8 +742,6 @@ long	v;
 	int		op;
 	token_type	*b1, *p1, *p2;
 	token_type	*ep;
-	int		zflag;
-	int		j, k;
 
 	partial_flag = false;
 	ufactor(tlhs, &n_tlhs);
@@ -816,8 +815,6 @@ lonely:
 		if (len2 > len1)
 			foundp = false;
 	}
-	zflag = (n_trhs == 1 && trhs[0].kind == CONSTANT
-	    && trhs[0].token.constant == 0.0);
 	b1 = p1 + 1;
 	op = p1->token.operatr;
 	if (op == POWER && b1->level == 1
@@ -850,24 +847,6 @@ lonely:
 			break;
 		}
 	}
-#if	!SILENT
-	if (op == TIMES && zflag) {
-		foundp = false;
-		k = b1 - tlhs;
-		for (j = k + 1;; j += 2) {
-			if (tlhs[j-1].kind == VARIABLE
-			    && tlhs[j-1].token.variable == v)
-				foundp = true;
-			if (j >= n_tlhs || tlhs[j].level <= 1)
-				break;
-		}
-		if (foundp) {
-			printf(_("Removing possible solution: \""));
-			list_proc(b1, j - k, false);
-			printf(" = 0\".\n");
-		}
-	}
-#endif
 	if (!g_of_f(op, b1, tlhs, &n_tlhs, trhs, &n_trhs))
 		return false;
 	list_tdebug(2);
@@ -992,7 +971,7 @@ long	v;
 		opx1 = MINUS;
 	else
 		opx1 = PLUS;
-#if	!SILENT
+#if	false
 	if (high_power > 1.0) {
 		list_var(v, false);
 		printf(_("The highest power of (%s) in equation is %g\n"), var_str, high_power);
@@ -1214,7 +1193,7 @@ big_bbreak:
 	} else if (high_power == 4.0) {
 		printf(_("Equation was biquadratic.\n"));
 	} else {
-		printf(_("Equation was quasi-quadratic.\n"));
+		printf(_("Equation was a degree %g quadratic.\n"), high_power);
 	}
 #endif
 	return true;
@@ -1486,9 +1465,7 @@ int		*side2np;
 {
 	token_type	*p1, *ep;
 
-#if	!SILENT
-	printf(_("Taking the reciprocal of both sides of the equation...\n"));
-#endif
+	debug_string(0, _("Taking the reciprocal of both sides of the equation..."));
 	if (*side1np + 2 > n_tokens || *side2np + 2 > n_tokens) {
 		error_huge();
 	}
@@ -1532,7 +1509,7 @@ next_espace()
 		if (i > n_equations) {
 #if	!SILENT
 			printf(_("Out of free equation spaces.\n"));
-			printf(_("Use the 'clear' command on unnecessary equations and try again.\n"));
+			printf(_("Please use the \"clear\" command on unnecessary equations and try again.\n"));
 #endif
 			longjmp(jmp_save, 3);
 		}
