@@ -47,30 +47,41 @@ sum_product(cp, product_flag)
 char	*cp;
 int	product_flag;
 {
-	int	i, j;
-	long	v;
-	char	*cp1;
-	double	d1, d2;
-	int	result_en;
-	int	n;
-	int	count_down;
+	int		i, j;
+	long		v;
+	char		*cp1;
+	double		d1, d2;
+	int		result_en;
+	int		n, ns;
+	token_type	*dest, *source;
+	int		count_down;
 
 	v = 0L;
 	if (notdefined(cur_equation)) {
 		return false;
 	}
+	result_en = next_espace();
+	if (n_rhs[cur_equation]) {
+		ns = n_rhs[cur_equation];
+		source = rhs[cur_equation];
+		dest = rhs[result_en];
+	} else {
+		ns = n_lhs[cur_equation];
+		source = lhs[cur_equation];
+		dest = lhs[result_en];
+	}
 	if (*cp == '\0') {
-		for (j = 0; j < n_rhs[cur_equation]; j++) {
-			if (rhs[cur_equation][j].kind == VARIABLE) {
-				if ((rhs[cur_equation][j].token.variable & VAR_MASK) <= SIGN)
+		for (j = 0; j < ns; j++) {
+			if (source[j].kind == VARIABLE) {
+				if ((source[j].token.variable & VAR_MASK) <= SIGN)
 					continue;
 				if (v) {
-					if (v != rhs[cur_equation][j].token.variable) {
+					if (v != source[j].token.variable) {
 						v = 0L;
 						break;
 					}
 				} else {
-					v = rhs[cur_equation][j].token.variable;
+					v = source[j].token.variable;
 				}
 			}
 		}
@@ -84,11 +95,10 @@ int	product_flag;
 			return false;
 		}
 	}
-	if (!found_var(rhs[cur_equation], n_rhs[cur_equation], v)) {
+	if (!found_var(source, ns, v)) {
 		printf(_("Variable not found.\n"));
 		return false;
 	}
-	result_en = next_espace();
 	if (*cp) {
 		cp1 = cp;
 	} else {
@@ -98,8 +108,8 @@ int	product_flag;
 			return false;
 	}
 	d1 = strtod(cp1, &cp);
-	if (cp1 == cp || fmod(d1, 1.0) != 0.0) {
-		printf(_("Error: not an integer.\n"));
+	if (cp1 == cp) {
+		printf(_("Invalid number.\n"));
 		return false;
 	}
 	cp = skip_space(cp);
@@ -111,27 +121,27 @@ int	product_flag;
 			return false;
 	}
 	d2 = strtod(cp1, &cp);
-	if (cp1 == cp || fmod(d2, 1.0) != 0.0) {
-		printf(_("Error: not an integer.\n"));
+	if (cp1 == cp) {
+		printf(_("Invalid number.\n"));
 		return false;
 	}
 	if (extra_garbage(cp))
 		return false;
 	count_down = (d2 < d1);
-	rhs[result_en][0].kind = CONSTANT;
-	rhs[result_en][0].level = 1;
+	dest[0].kind = CONSTANT;
+	dest[0].level = 1;
 	if (product_flag) {
-		rhs[result_en][0].token.constant = 1.0;
+		dest[0].token.constant = 1.0;
 	} else {
-		rhs[result_en][0].token.constant = 0.0;
+		dest[0].token.constant = 0.0;
 	}
 	n = 1;
 	for (; count_down ? (d1 >= d2) : (d1 <= d2); count_down ? (d1 -= 1.0) : (d1 += 1.0)) {
-		if (n + 1 + n_rhs[cur_equation] > n_tokens) {
+		if (n + 1 + ns > n_tokens) {
 			error_huge();
 		}
-		blt(tlhs, rhs[cur_equation], n_rhs[cur_equation] * sizeof(token_type));
-		n_tlhs = n_rhs[cur_equation];
+		blt(tlhs, source, ns * sizeof(token_type));
+		n_tlhs = ns;
 		for (i = 0; i < n_tlhs; i += 2) {
 			if (tlhs[i].kind == VARIABLE && tlhs[i].token.variable == v) {
 				tlhs[i].kind = CONSTANT;
@@ -142,23 +152,27 @@ int	product_flag;
 			tlhs[i].level++;
 		}
 		for (i = 0; i < n; i++) {
-			rhs[result_en][i].level++;
+			dest[i].level++;
 		}
-		rhs[result_en][n].kind = OPERATOR;
-		rhs[result_en][n].level = 1;
+		dest[n].kind = OPERATOR;
+		dest[n].level = 1;
 		if (product_flag) {
-			rhs[result_en][n].token.operatr = TIMES;
+			dest[n].token.operatr = TIMES;
 		} else {
-			rhs[result_en][n].token.operatr = PLUS;
+			dest[n].token.operatr = PLUS;
 		}
 		n++;
-		blt(&rhs[result_en][n], tlhs, n_tlhs * sizeof(token_type));
+		blt(&dest[n], tlhs, n_tlhs * sizeof(token_type));
 		n += n_tlhs;
-		calc_simp(rhs[result_en], &n);
+		calc_simp(dest, &n);
+	}
+	if (n_rhs[cur_equation]) {
+		n_rhs[result_en] = n;
+		blt(lhs[result_en], lhs[cur_equation], n_lhs[cur_equation] * sizeof(token_type));
+		n_lhs[result_en] = n_lhs[cur_equation];
+	} else {
+		n_lhs[result_en] = n;
 	}
-	n_rhs[result_en] = n;
-	blt(lhs[result_en], lhs[cur_equation], n_lhs[cur_equation] * sizeof(token_type));
-	n_lhs[result_en] = n_lhs[cur_equation];
 	list_sub(result_en);
 	return true;
 }
@@ -401,7 +415,7 @@ char	*cp;
 	high_prec = true;
 	add_history(list_equation(i, false));
 	high_prec = false;
-	printf(_("Equation pushed.  Hit the UP key to access.\n"));
+	debug_string(0, _("Equation pushed.  Hit the UP key to access."));
 	return true;
 }
 #endif
@@ -607,11 +621,9 @@ char	*cp;
 	int	i1;
 	char	exists[N_EQUATIONS];
 
-	if (!get_range(&cp, &i, &j)) {
+	if (!get_range_eol(&cp, &i, &j)) {
 		return false;
 	}
-	if (extra_garbage(cp))
-		return false;
 	for (i1 = 0; i1 < N_EQUATIONS; i1++) {
 		exists[i1] = false;
 	}
@@ -650,6 +662,7 @@ int	imag_flag;
 		return false;
 	}
 	j = next_espace();
+	simp_loop(rhs[i], &n_rhs[i]);
 	uf_simp(rhs[i], &n_rhs[i]);
 	factorv(rhs[i], &n_rhs[i], IMAGINARY);
 	partial_flag = false;
@@ -820,12 +833,11 @@ char	*cp;
 		if (cp == NULL) {
 			return false;
 		}
-		iterations = atoi(cp);
+		iterations = strtol(cp, &cp, 10);
 		if (iterations <= 0) {
 			printf(_("Invalid number of iterations.\n"));
 			return false;
 		}
-		cp = skip_num(cp);
 		if (extra_garbage(cp))
 			return false;
 	}
@@ -963,11 +975,9 @@ char	*cp;
 	if (is_all(cp)) {
 		clear_all();
 	} else {
-		if (!get_range(&cp, &i, &j)) {
+		if (!get_range_eol(&cp, &i, &j)) {
 			return false;
 		}
-		if (extra_garbage(cp))
-			return false;
 		for (; i <= j; i++) {
 			n_lhs[i] = 0;
 		}
@@ -982,12 +992,10 @@ int	*diff_signp;
 {
 	int	rv;
 
-	sign_flag = true;
 	rv = se_compare(rhs[i], n_rhs[i], rhs[i], n_rhs[i], diff_signp);
-	sign_flag = false;
 	if (!rv || *diff_signp) {
 		printf(_("Error in compare function or too many terms to compare!\n"));
-		longjmp(jmp_save, 2);
+		return false;
 	}
 	sign_flag = true;
 	rv = se_compare(rhs[i], n_rhs[i], rhs[j], n_rhs[j], diff_signp);
@@ -999,7 +1007,7 @@ int	*diff_signp;
  * The compare command.
  */
 int
-compare(cp)
+compare_cmd(cp)
 char	*cp;
 {
 	int		i, j;
@@ -1007,8 +1015,8 @@ char	*cp;
 	int		diff_sign;
 	int		already_solved;
 
-	i = atoi(cp) - 1;
-	cp = skip_num(cp);
+	i = strtol(cp, &cp, 10) - 1;
+	cp = skip_space(cp);
 	if (notdefined(i)) {
 		return false;
 	}
@@ -1030,11 +1038,15 @@ char	*cp;
 		if (compare_rhs(i, j, &diff_sign)) {
 			goto times_neg1;
 		}
-		printf(_("Simplifying both equations...\n"));
-		simpa_side(rhs[i], &n_rhs[i], false, false);
-		list_sub(i);
-		simpa_side(rhs[j], &n_rhs[j], false, false);
-		list_sub(j);
+		debug_string(0, _("Simplifying both equations..."));
+		simpa_side(rhs[i], &n_rhs[i], false, true);
+		simpa_side(rhs[j], &n_rhs[j], false, true);
+#if	!SILENT
+		if (debug_level >= 0) {
+			list_sub(i);
+			list_sub(j);
+		}
+#endif
 		if (compare_rhs(i, j, &diff_sign)) {
 			goto times_neg1;
 		}
@@ -1044,7 +1056,7 @@ char	*cp;
 			goto times_neg1;
 		}
 	}
-	printf(_("Solving both equations for zero and unfactoring...\n"));
+	debug_string(0, _("Solving both equations for zero and unfactoring..."));
 	want.level = 1;
 	want.kind = CONSTANT;
 	want.token.constant = 0.0;
@@ -1059,9 +1071,9 @@ char	*cp;
 		printf(_("Equations are identical.\n"));
 		return true;
 	}
-	printf(_("Simplifying both equations...\n"));
-	simpa_side(rhs[i], &n_rhs[i], false, false);
-	simpa_side(rhs[j], &n_rhs[j], false, false);
+	debug_string(0, _("Simplifying both equations..."));
+	simpa_side(rhs[i], &n_rhs[i], false, true);
+	simpa_side(rhs[j], &n_rhs[j], false, true);
 	if (compare_rhs(i, j, &diff_sign)) {
 		printf(_("Equations are identical.\n"));
 		return true;
@@ -1102,11 +1114,11 @@ int
 divide_cmd(cp)
 char	*cp;
 {
-	long	v, v_tmp;
-	int	i, j;
-	int	nl, nr;
-	double	d1, d2;
-	double	d3, d4;
+	long		v, v_tmp;
+	int		i, j;
+	int		nl, nr;
+	double		d1, d2;
+	double		d3, d4;
 	complexs	c1, c2, c3;
 
 	v = 0;
@@ -1263,7 +1275,7 @@ next_var:
 			return false;
 		}
 		if (i == cur_equation) {
-			printf(_("Error: source and destination equations are the same equation.\n"));
+			printf(_("Error: source and destination are the same.\n"));
 			return false;
 		}
 		if (!elim_sub(i, v))
@@ -1384,16 +1396,16 @@ long	v;
 	token_type	want;
 
 #if	!SILENT
-	list_var(v, false);
-	printf(_("Solving equation #%d for (%s)...\n"), i + 1, var_str);
+	if (debug_level >= 0) {
+		list_var(v, false);
+		printf(_("Solving equation #%d for (%s)...\n"), i + 1, var_str);
+	}
 #endif
 	want.level = 1;
 	want.kind = VARIABLE;
 	want.token.variable = v;
 	if (!solve_sub(&want, 1, lhs[i], &n_lhs[i], rhs[i], &n_rhs[i])) {
-#if	!SILENT
-		printf(_("Solve failed.\n"));
-#endif
+		debug_string(0, _("Solve failed."));
 		return false;
 	}
 	subst_var_with_exp(rhs[cur_equation], &n_rhs[cur_equation], rhs[i], n_rhs[i], v);
@@ -1417,11 +1429,9 @@ char	*cp;
 	if (factor_flag = (strcmp_tospace(cp, "factor") == 0)) {
 		cp = skip_param(cp);
 	}
-	if (!get_range(&cp, &i, &j)) {
+	if (!get_range_eol(&cp, &i, &j)) {
 		return false;
 	}
-	if (extra_garbage(cp))
-		return false;
 	blt(save_save, jmp_save, sizeof(jmp_save));
 	if (setjmp(jmp_save) != 0) {
 		i++;
@@ -1454,11 +1464,9 @@ char	*cp;
 	if (export_flag = (strcmp_tospace(cp, "export") == 0)) {
 		cp = skip_param(cp);
 	}
-	if (!get_range(&cp, &i, &j)) {
+	if (!get_range_eol(&cp, &i, &j)) {
 		return false;
 	}
-	if (extra_garbage(cp))
-		return false;
 	for (; i <= j; i++) {
 		list1_sub(i, export_flag);
 	}
@@ -1487,11 +1495,9 @@ char	*cp;
 		cp = skip_param(cp);
 		int_flag = true;
 	}
-	if (!get_range(&cp, &i, &j)) {
+	if (!get_range_eol(&cp, &i, &j)) {
 		return false;
 	}
-	if (extra_garbage(cp))
-		return false;
 	for (; i <= j; i++) {
 		if (n_lhs[i] > 0) {
 			if (n_rhs[i] == 0 || n_lhs[i] != 1 || lhs[i][0].kind != VARIABLE) {
@@ -1666,27 +1672,24 @@ char	*cp;
 	poly_flag = false;
 	quick_flag = false;
 	symb = false;
-check_again:
-	if (strncasecmp(cp, "poly", 4) == 0) {
-		poly_flag = true;
-		cp = skip_param(cp);
-		goto check_again;
+	for (;; cp = skip_param(cp)) {
+		if (strncasecmp(cp, "poly", 4) == 0) {
+			poly_flag = true;
+			continue;
+		}
+		if (strncasecmp(cp, "symbolic", 4) == 0) {
+			symb = true;
+			continue;
+		}
+		if (strncasecmp(cp, "quick", 4) == 0) {
+			quick_flag = true;
+			continue;
+		}
+		break;
 	}
-	if (strncasecmp(cp, "symbolic", 4) == 0) {
-		symb = true;
-		cp = skip_param(cp);
-		goto check_again;
-	}
-	if (strncasecmp(cp, "quick", 4) == 0) {
-		quick_flag = true;
-		cp = skip_param(cp);
-		goto check_again;
-	}
-	if (!get_range(&cp, &i, &j)) {
+	if (!get_range_eol(&cp, &i, &j)) {
 		return false;
 	}
-	if (extra_garbage(cp))
-		return false;
 	symb_flag = symb;
 	for (; i <= j; i++) {
 		if (n_lhs[i]) {
@@ -1738,10 +1741,6 @@ next_number:
 			}
 		}
 		printf("\n");
-		if (d != multiply_out_unique()) {
-			printf(_("Error in computation!\n"));
-			return false;
-		}
 		if (*cp)
 			goto next_number;
 		return true;
@@ -1757,7 +1756,7 @@ next_number:
 			}
 		}
 		for (i1 = i; i1 <= j; i1++) {
-			if (n_lhs[i1] > 0) {
+			if (n_lhs[i1]) {
 				simpv_side(lhs[i1], &n_lhs[i1], v);
 				simpv_side(rhs[i1], &n_rhs[i1], v);
 			}
@@ -1782,14 +1781,12 @@ char	*cp;
 	if (fully_flag = (strncasecmp(cp, "fully", 4) == 0)) {
 		cp = skip_param(cp);
 	}
-	if (!get_range(&cp, &i, &j)) {
+	if (!get_range_eol(&cp, &i, &j)) {
 		return false;
 	}
-	if (extra_garbage(cp))
-		return false;
 	partial_flag = !fully_flag;
 	for (; i <= j; i++) {
-		if (n_lhs[i] > 0) {
+		if (n_lhs[i]) {
 			uf_simp(lhs[i], &n_lhs[i]);
 			uf_simp(rhs[i], &n_rhs[i]);
 			list_sub(i);
@@ -1815,7 +1812,7 @@ char	*cp;
  * The read command.
  */
 int
-read_in(cp)
+read_cmd(cp)
 char	*cp;
 {
 	int	rv;
@@ -1832,7 +1829,7 @@ char	*cp;
 	if (fp == NULL) {
 		fp = fopen(cp, "r");
 		if (fp == NULL) {
-			printf(_("Can't open file \"%s\".\n"), cp);
+			printf(_("Can't open file \"%s\" or \"%s\".\n"), buf, cp);
 			return false;
 		}
 	}
@@ -1970,9 +1967,8 @@ edit_again:
 	}
 	default_color();
 	clear_all();
-	if (!read_in(cp)) {
-		printf(_("Prepare to run the editor.\n"));
-		if (pause_cmd("")) {
+	if (!read_cmd(cp)) {
+		if (pause_cmd("Prepare to run the editor")) {
 			goto edit_again;
 		}
 	}
@@ -1997,7 +1993,7 @@ char	*cp;
 	}
 	if (access(cp, 0) == 0) {
 		do {
-			snprintf(prompt_str, sizeof(prompt_str), _("'%s' exists.  Overwrite (Y/N)? "), cp);
+			snprintf(prompt_str, sizeof(prompt_str), _("\"%s\" exists.  Overwrite (Y/N)? "), cp);
 			if ((cp1 = getstring((char *) trhs, n_tokens * sizeof(token_type))) == NULL) {
 				return false;
 			}
@@ -2010,7 +2006,7 @@ char	*cp;
 	}
 	fp = fopen(cp, "w");
 	if (fp == NULL) {
-		printf(_("Can't create file '%s'.\n"), cp);
+		printf(_("Can't create file \"%s\".\n"), cp);
 		return false;
 	}
 	gfp = fp;
@@ -2021,7 +2017,7 @@ char	*cp;
 	if (fclose(fp))
 		rv = false;
 	if (rv) {
-		printf(_("All equations saved in file: '%s'.\n"), cp);
+		printf(_("All equations saved in file: \"%s\".\n"), cp);
 	} else {
 		printf(_("Error encountered while saving equations.\n"));
 	}
@@ -2032,6 +2028,7 @@ char	*cp;
 /*
  * Get default equation number from a command parameter string.
  * The equation number must be the only parameter.
+ *
  * Return -1 on error.
  */
 int
@@ -2043,8 +2040,7 @@ char	*cp;
 	if (*cp == '\0')
 		i = cur_equation;
 	else {
-		i = atoi(cp) - 1;
-		cp = skip_num(cp);
+		i = strtol(cp, &cp, 10) - 1;
 		if (extra_garbage(cp))
 			return -1;
 	}
@@ -2056,6 +2052,8 @@ char	*cp;
 
 /*
  * Get an expression from the user.
+ * The prompt must be previously copied into the global prompt_str[].
+ *
  * Return true if successful.
  */
 int
@@ -2083,6 +2081,7 @@ int		*np;
 
 /*
  * Prompt for a variable from the user.
+ *
  * Return true if successful.
  */
 int
@@ -2125,7 +2124,7 @@ int	i;
 }
 
 /*
- * Common routine to output a prompt
+ * Common routine to output the prompt in prompt_str[]
  * and get a line of input from stdin.
  * All Mathomatic input comes from this routine,
  * except for file reading.
