@@ -341,14 +341,13 @@ char	*cp;
 			}
 		}
 		if (*cp) {
-			order = atoi(cp);
-			cp = skip_num(cp);
+			order = strtol(cp, &cp, 10);
+		}
+		if (extra_garbage(cp)) {
+			return false;
 		}
 	}
-	if (extra_garbage(cp)) {
-		return false;
-	}
-	if (order < 1) {
+	if (order <= 0) {
 		printf(_("Order out of range.\n"));
 		return false;
 	}
@@ -367,6 +366,7 @@ char	*cp;
 	for (j = 0; j < order; j++) {
 		if (!differentiate(dest, np, v)) {
 			printf(_("Differentiation failed.\n"));
+			*np = 0;
 			return false;
 		}
 		simpa_side(dest, np, false, true);
@@ -530,7 +530,8 @@ oops:
 		printf(_("Derivatives will be taken until they reach zero.\n"));
 		order = INT_MAX;
 	} else {
-		if (!isascii(*cp1) || !isdigit(*cp1) || (order = atoi(cp1)) < 0) {
+		order = strtol(cp1, &cp, 10);
+		if (cp == cp1 || order < 0) {
 			printf(_("Positive integer required.\n"));
 			return false;
 		}
