@@ -326,18 +326,17 @@ char	*cp;
 			return false;
 		}
 		if (*cp) {
-			order = atoi(cp);
-			cp = skip_num(cp);
+			order = strtol(cp, &cp, 10);
+		}
+		if (extra_garbage(cp)) {
+			return false;
 		}
 	} else {
 		if (!prompt_var(&v)) {
 			return false;
 		}
 	}
-	if (extra_garbage(cp)) {
-		return false;
-	}
-	if (order < 1) {
+	if (order <= 0) {
 		printf(_("Order out of range.\n"));
 		return false;
 	}
@@ -350,6 +349,7 @@ char	*cp;
 	for (j = 0; j < order; j++) {
 		if (!integrate(dest, np, v)) {
 			printf(_("Integration failed.\n"));
+			*np = 0;
 			return false;
 		}
 		simp_loop(dest, np);
@@ -729,13 +729,27 @@ int
 laplace_cmd(cp)
 char	*cp;
 {
-	int	i;
-	long	v;
-	int	inverse_flag;
+	int		i;
+	long		v;
+	int		inverse_flag;
+	token_type	*source, *dest;
+	int		*nps, *np;
 
 	if (notdefined(cur_equation)) {
 		return false;
 	}
+	i = next_espace();
+	if (n_rhs[cur_equation]) {
+		source = rhs[cur_equation];
+		nps = &n_rhs[cur_equation];
+		dest = rhs[i];
+		np = &n_rhs[i];
+	} else {
+		source = lhs[cur_equation];
+		nps = &n_lhs[cur_equation];
+		dest = lhs[i];
+		np = &n_lhs[i];
+	}
 	inverse_flag = (strcmp_tospace(cp, "inverse") == 0);
 	if (inverse_flag) {
 		cp = skip_param(cp);
@@ -753,34 +767,36 @@ char	*cp;
 	if (extra_garbage(cp)) {
 		return false;
 	}
-	i = next_espace();
 	partial_flag = false;
-	uf_simp(rhs[cur_equation], &n_rhs[cur_equation]);
+	uf_simp(source, nps);
 	partial_flag = true;
-	factorv(rhs[cur_equation], &n_rhs[cur_equation], v);
-	blt(rhs[i], rhs[cur_equation], n_rhs[cur_equation] * sizeof(token_type));
-	n_rhs[i] = n_rhs[cur_equation];
+	factorv(source, nps, v);
+	blt(dest, source, *nps * sizeof(token_type));
+	*np = *nps;
 	if (inverse_flag) {
-		if (!poly_in_v(rhs[i], n_rhs[i], v, true)
-		    || !inv_laplace(rhs[i], &n_rhs[i], v)) {
+		if (!poly_in_v(dest, *np, v, true)
+		    || !inv_laplace(dest, np, v)) {
 			printf(_("Inverse Laplace failed.\n"));
+			*np = 0;
 			return false;
 		}
 	} else {
-		if (!laplace(rhs[i], &n_rhs[i], v)) {
+		if (!laplace(dest, np, v)) {
 			printf(_("Laplace failed.\n"));
+			*np = 0;
 			return false;
 		}
 	}
-	simp_loop(rhs[i], &n_rhs[i]);
-	blt(lhs[i], lhs[cur_equation], n_lhs[cur_equation] * sizeof(token_type));
-	n_lhs[i] = n_lhs[cur_equation];
+	if (n_rhs[cur_equation]) {
+		blt(lhs[i], lhs[cur_equation], n_lhs[cur_equation] * sizeof(token_type));
+		n_lhs[i] = n_lhs[cur_equation];
+	}
 	cur_equation = i;
+	simp_loop(dest, np);
 	list_sub(cur_equation);
 	return true;
 }
 
-#if	NUMERICAL_INTEGRATE
 /*
  * Numerical integrate command.
  */
@@ -798,12 +814,16 @@ char	*cp;
 	int		first_size;
 	token_type	*ep;
 	int		trap_flag;
+	int		singularity;
 
 	iterations = 1000;	/* must be even */
-	trap_flag = false;
 	if (notdefined(cur_equation)) {
 		return false;
 	}
+	trap_flag = (strncasecmp(cp, "trap", 4) == 0);
+	if (trap_flag) {
+		cp = skip_param(cp);
+	}
 	if (*cp == '\0') {
 		if (!prompt_var(&v))
 			return false;
@@ -812,10 +832,9 @@ char	*cp;
 		if (cp == NULL) {
 			return false;
 		}
-		if (strncmp(cp, "trap", 4) == 0) {
+		if (strncasecmp(cp, "trap", 4) == 0) {
 			trap_flag = true;
-		} else if (*cp) {
-			printf(_("This program will only integrate with respect to one variable.\n"));
+		} else if (extra_garbage(cp)) {
 			return false;
 		}
 	}
@@ -824,6 +843,21 @@ char	*cp;
 	if (er) {
 		printf(_("Warning: Current equation is not a solved equation.\n"));
 	}
+	singularity = false;
+	for (j = 1; j < n_rhs[cur_equation]; j += 2) {
+		if (rhs[cur_equation][j].token.operatr == DIVIDE) {
+			level = rhs[cur_equation][j].level;
+			for (k = j + 1; k < n_rhs[cur_equation] && rhs[cur_equation][k].level >= level; k++) {
+				if (rhs[cur_equation][k].kind == VARIABLE
+				    && rhs[cur_equation][k].token.variable == v) {
+					singularity = true;
+				}
+			}
+		}
+	}
+	if (singularity) {
+		printf(_("Warning: Singularity detected, result of numerical integration may be wrong.\n"));
+	}
 	my_strlcpy(prompt_str, _("Enter lower bound: "), sizeof(prompt_str));
 	if (!get_expr(tlhs, &n_tlhs)) {
 		return false;
@@ -959,9 +993,11 @@ char	*cp;
 			}
 			n_rhs[i] += 2;
 		}
+		in_calc_cmd = true;
 		elim_loop(rhs[i], &n_rhs[i]);
 		ufactor(rhs[i], &n_rhs[i]);
 		simp_divide(rhs[i], &n_rhs[i], 0L);
+		in_calc_cmd = false;
 		if (j > 0) {
 			if (j == 1) {
 				first_size = n_rhs[i];
@@ -1024,4 +1060,3 @@ char	*cp;
 	list_sub(cur_equation);
 	return true;
 }
-#endif
