@@ -11,9 +11,6 @@
 #include "../includes.h"
 #include <signal.h>
 
-void		fphandler(int sig);
-void		exit_program(int exit_value);
-
 int
 main(int argc, char **argv)
 {
@@ -31,7 +28,6 @@ main(int argc, char **argv)
 		clean_up();	/* Mathomatic processing was interrupted, so do a clean up. */
 		/* Handle longjmp() errors here. */
 	} else {
-		/* All signal() initialization goes here. */
 		signal(SIGFPE, fphandler);
 	}
 	/* Mathomatic is ready for use. */
