@@ -1,5 +1,5 @@
 /*
- * Algebraic manipulator expression display routines.
+ * Algebraic manipulator expression and equation display routines.
  *
  * Copyright (c) 1996 George Gesslein II.
  */
@@ -10,10 +10,12 @@ static int	flist_recurse();
 
 static int	cur_color = -1;
 
+#if	!NO_COLOR
 /* ANSI color code array */
 static int	carray[] = {
 	32, 33, 31, 35, 34, 36, 37
 };
+#endif
 
 /* HTML color array */
 static char	*html_carray[] = {
@@ -35,10 +37,16 @@ reset_attr()
 	if (html_flag) {
 		printf("</font>");
 	} else if (color_flag) {
+#if	!NO_COLOR
 		printf("\033[0m");
+#endif
 	}
 }
 
+/*
+ * Set the current color on the display.
+ * Range is 0 to INT_MAX.
+ */
 set_color(color)
 int	color;
 {
@@ -53,7 +61,9 @@ int	color;
 			printf("<font color=\"%s\">", html_carray[color%ARR_CNT(html_carray)]);
 		}
 	} else if (color_flag) {
+#if	!NO_COLOR
 		printf("\033[%dm", carray[color%ARR_CNT(carray)]);
+#endif
 	}
 	cur_color = color;
 }
