@@ -3,13 +3,13 @@
  */
 
 int		clear(), quit(), list_cmd(), simplify(), help(), eliminate();
-int		unfactor(), compare(), extrema_cmd();
-int		read_in(), flist_cmd(), calculate();
+int		unfactor(), compare_cmd(), extrema_cmd();
+int		read_cmd(), flist_cmd(), calculate();
 int		factor(), derivative(), replace();
 int		save(), print_cmd(), taylor(), limit_cmd();
 int		copy_cmd(), divide_cmd(), pause_cmd();
 int		edit(), real(), imaginary(), tally();
-int		roots(), set_cmd(), code_cmd(), optimize_cmd(), push_cmd();
+int		roots_cmd(), set_cmd(), code_cmd(), optimize_cmd(), push_cmd();
 int		sum_cmd(), product_cmd(), integrate_cmd(), nintegrate(), laplace_cmd();
 char		*skip_space(), *skip_param();
 char		*skip_num();
