@@ -6,7 +6,7 @@
 
 #include "includes.h"
 
-#define	MAX_COMPARE_TERMS	200	/* Max no. of terms on same level that can be compared. */
+#define	MAX_COMPARE_TERMS	400	/* Max no. of terms on same level that can be compared.  Increases stack usage. */
 
 static int	sp2_sub();
 static int	sp_sub();
@@ -32,19 +32,19 @@ int	n;	/* equation number */
  * For quick, mid-range simplification of an expression.
  */
 simp_side(equation, np)
-token_type	*equation;
-int		*np;
+token_type	*equation;	/* expression */
+int		*np;		/* pointer to length of expression */
 {
 	simp_ssub(equation, np, 0L, 0.0, true, true, 1);
 }
 
 /*
- * This function is used by the solver.
+ * This function is the mid-range simplifier used by the solver.
  */
 simps_side(equation, np, zsolve)
-token_type	*equation;
-int		*np;
-int		zsolve;
+token_type	*equation;	/* expression */
+int		*np;		/* pointer to length of expression */
+int		zsolve;		/* true if we are solving for zero */
 {
 	do {
 		simp_ssub(equation, np, 0L, 0.0, !zsolve, true, 2);
@@ -132,7 +132,7 @@ int		poly_flag, quick_flag;
 	} while (uf_power(equation, np) || poly_factor(equation, np, true));
 	simp_ssub(equation, np, 0L, 0.0, true, true, 1);
 	rationalize(equation, np);
-	unsimp_power(equation, np, false);
+	unsimp_power(equation, np);
 	uf_times(equation, np);
 	simp_loop(equation, np);
 	uf_repeat(equation, np);
@@ -141,19 +141,21 @@ int		poly_flag, quick_flag;
 	} while (mod_simp(equation, np));
 /* Here we try to simplify out unnecessary negative constants and imaginary numbers */
 	simp_i(equation, np);
-	unsimp_power(equation, np, false);
+	unsimp_power(equation, np);
 	uf_times(equation, np);
 	simp_ssub(equation, np, 0L, 1.0, true, true, 1);
-	unsimp_power(equation, np, false);
+	unsimp_power(equation, np);
 	uf_neg_help(equation, np);
 	uf_times(equation, np);
 	do {
 		do {
-			simp_ssub(equation, np, 0L, 1.0, false, true, 1);
-		} while (uf_power(equation, np));
-	} while (super_factor(equation, np, false));
+			do {
+				simp_ssub(equation, np, 0L, 1.0, false, true, 1);
+			} while (uf_power(equation, np));
+		} while (polydiv_simp(equation, np));
+	} while (super_factor(equation, np, 2));
 	side_debug(2, equation, *np);
-	unsimp_power(equation, np, false);
+	unsimp_power(equation, np);
 	uf_times(equation, np);
 	simp_loop(equation, np);
 	uf_pplus(equation, np);
@@ -200,7 +202,7 @@ do_quick:
 	flag = false;
 try_again:
 	/* divide top and bottom of fractions by any polynomial GCD found */
-	if (polydiv_simp(equation, np)) {
+	while (polydiv_simp(equation, np)) {
 		flag = false;
 		simpb_side(equation, np, true, 1);
 	}
@@ -222,7 +224,7 @@ try_again:
 	partial_flag = true;
 	simp_constant_power(equation, np);
 	simp_ssub(equation, np, 0L, 1.0, true, true, 1);
-	unsimp_power(equation, np, false);
+	unsimp_power(equation, np);
 	make_fractions(equation, np);
 	factor_power(equation, np);
 	uf_tsimp(equation, np);
@@ -1047,18 +1049,14 @@ double	k2;
 			d = -(*k1p);
 		else
 			d = *k1p;
-#if	EPSILON
 		d1 = fabs(d) * epsilon;
-#endif
 		if (op2 == PLUS) {
 			d += k2;
 		} else {
 			d -= k2;
 		}
-#if	EPSILON
 		if (fabs(d) < d1)
 			d = 0.0;
-#endif
 		if (op1 == 0) {
 			*k1p = d;
 		} else {
@@ -1085,13 +1083,11 @@ double	k2;
 				*k1p = *k1p / k2;
 			}
 		}
-#if	EPSILON
 		d1 = fabs(*k1p) * small_epsilon;
 		d = my_round(*k1p);
 		if (d != 0.0 && fabs(d - *k1p) < d1) {
 			*k1p = d;
 		}
-#endif
 		break;
 	case MODULUS:
 #if	false	/* If fmod() always works.  It doesn't. */
@@ -1302,11 +1298,7 @@ int		*np;
 						modified = true;
 						continue;
 					}
-#if	EPSILON
 					if (fabs(d - 1.0) <= epsilon) {
-#else
-					if (d == 1.0) {
-#endif
 						blt(p1 - 1, p1 + 1, (int) ((char *) ep - (char *) (p1 + 1)));
 						*np -= 2;
 						modified = true;
@@ -1404,11 +1396,7 @@ int		*np;
 						modified = true;
 						continue;
 					}
-#if	EPSILON
 					if (fabs((p1 + 1)->token.constant - 1.0) <= epsilon) {
-#else
-					if ((p1 + 1)->token.constant == 1.0) {
-#endif
 						blt(p1, p1 + 2, (int) ((char *) ep - (char *) (p1 + 2)));
 						*np -= 2;
 						modified = true;
@@ -1481,7 +1469,6 @@ int		n2, l2, *diff_signp;
 				*diff_signp = true;
 				return true;
 			}
-#if	EPSILON
 			c1 = p1->token.constant;
 			c2 = p2->token.constant;
 			d1 = fabs(c1) * epsilon;
@@ -1492,7 +1479,6 @@ int		n2, l2, *diff_signp;
 				*diff_signp = true;
 				return true;
 			}
-#endif
 		}
 		return false;
 	}
@@ -1542,11 +1528,7 @@ int		n2, l2, *diff_signp;
 		if (p1->kind == CONSTANT
 		    && p1->level == l1
 		    && op1 == TIMES) {
-#if	EPSILON
 			if (fabs(fabs(p1->token.constant) - 1.0) <= epsilon) {
-#else
-			if (fabs(p1->token.constant) == 1.0) {
-#endif
 				if (!compare_recurse(p1 + 2, n1 - 2, min_level(p1 + 2, n1 - 2),
 				    p2, n2, l2, diff_signp))
 					return false;
@@ -1559,11 +1541,7 @@ int		n2, l2, *diff_signp;
 		if (p2->kind == CONSTANT
 		    && p2->level == l2
 		    && op2 == TIMES) {
-#if	EPSILON
 			if (fabs(fabs(p2->token.constant) - 1.0) <= epsilon) {
-#else
-			if (fabs(p2->token.constant) == 1.0) {
-#endif
 				if (!compare_recurse(p1, n1, l1,
 				    p2 + 2, n2 - 2, min_level(p2 + 2, n2 - 2), diff_signp))
 					return false;
@@ -1599,19 +1577,11 @@ no_op2:
 			if (i >= oc2) {
 				if ((op1 == TIMES || op1 == DIVIDE)
 				    && pv1->level == l1 && pv1->kind == CONSTANT) {
-#if	EPSILON
 					if (fabs(pv1->token.constant + 1.0) <= epsilon) {
-#else
-					if (pv1->token.constant == -1.0) {
-#endif
 						*diff_signp ^= true;
 						break;
 					}
-#if	EPSILON
 					if (fabs(pv1->token.constant - 1.0) <= epsilon) {
-#else
-					if (pv1->token.constant == 1.0) {
-#endif
 						break;
 					}
 				}
@@ -1677,19 +1647,11 @@ no_op2:
 		if (!used[i]) {
 			if ((op2 == TIMES || op2 == DIVIDE)
 			    && opa2[i]->level == l2 && opa2[i]->kind == CONSTANT) {
-#if	EPSILON
 				if (fabs(opa2[i]->token.constant + 1.0) <= epsilon) {
-#else
-				if (opa2[i]->token.constant == -1.0) {
-#endif
 					*diff_signp ^= true;
 					continue;
 				}
-#if	EPSILON
 				if (fabs(opa2[i]->token.constant - 1.0) <= epsilon) {
-#else
-				if (opa2[i]->token.constant == 1.0) {
-#endif
 					continue;
 				}
 			}
