@@ -1004,9 +1004,7 @@ store_code:
 					blt(&equation[i+n_tlhs+1], &equation[i+1+len1], (*np - (i + 1 + len1)) * sizeof(*equation));
 					*np += n_tlhs - len1;
 					blt(&equation[i+1], tlhs, n_tlhs * sizeof(*equation));
-#if	!SILENT
-					printf(_("Found polynomial Greatest Common Divisor.  Division simplified.\n"));
-#endif
+					debug_string(0, _("Found polynomial Greatest Common Divisor.  Division simplified."));
 					return true;
 				}
 				if (poly2_gcd(&equation[j], len2, &equation[i+1], len1, 0L)) {
@@ -1201,13 +1199,11 @@ next_thingy:
 						blt(&equation[j+n_tlhs], &equation[j+len2], (*np - (j + len2)) * sizeof(token_type));
 						*np -= len2 - n_tlhs;
 						blt(&equation[j], tlhs, n_tlhs * sizeof(token_type));
-#if	!SILENT
 						if (flag || power_flag) {
-							printf(_("Polynomial division successful.\n"));
+							debug_string(0, _("Polynomial division successful."));
 						} else {
-							printf(_("Smart division successful.\n"));
+							debug_string(0, _("Smart division successful."));
 						}
-#endif
 						side_debug(3, equation, *np);
 						return true;
 					}
